/* Copyright Abandoned 1996 TCX DataKonsult AB & Monty Program KB & Detron HB
   This file is public domain and comes with NO WARRANTY of any kind */

/* Routines to handle mallocing of results withs will be freed the same time */

#include <global.h>
#include <my_sys.h>
#include <m_string.h>
#include "mysql.h"

static uint const sql_alloc_extra=8192-MALLOC_OVERHEAD-sizeof(USED_MEM)-8;

void init_sql_alloc(MEM_ROOT *mem_root)
{
  mem_root->free=mem_root->used=0;
  mem_root->min_malloc=16;
}


gptr sql_alloc_root(MEM_ROOT *mem_root,unsigned int Size)
{
  uint get_size,max_left;
  gptr point;
  reg1 USED_MEM *next;
  reg2 USED_MEM **prev;

  Size= ALIGN_SIZE(Size);
  prev= &mem_root->free;
  max_left=0;
  for (next= *prev ; next && next->left < Size ; next= next->next)
  {
    if (next->left > max_left)
      max_left=next->left;
    prev= &next->next;
  }
  if (! next)
  {						/* Time to alloc new block */
    get_size= Size+ALIGN_SIZE(sizeof(USED_MEM));
    if (max_left*4 < sql_alloc_extra && get_size < sql_alloc_extra)
      get_size=sql_alloc_extra;			/* Normal alloc */

    if (!(next = (USED_MEM*) my_malloc(get_size,MYF(MY_WME))))
      return((gptr) 0);
    next->next= *prev;
    next->size= get_size;
    next->left= get_size-ALIGN_SIZE(sizeof(USED_MEM));
    *prev=next;
  }
  point= (gptr) ((char*) next+ (next->size-next->left));
  if ((next->left-= Size) < mem_root->min_malloc)
  {					/* Full. Remove block from list */
    *prev=next->next;
    next->next=mem_root->used;
    mem_root->used=next;
  }
  return(point);
}

	/* deallocate everything used by sql_alloc */

void sql_free(MEM_ROOT *root)
{
  reg1 USED_MEM *next,*old;
  DBUG_ENTER("sql_free");

  if (!root)
    DBUG_VOID_RETURN;
  for (next= root->used ; next ; )
  {
    old=next; next= next->next ;
    my_free((gptr) old,MYF(0));
  }
  for (next= root->free ; next ; )
  {
    old=next; next= next->next ;
    my_free((gptr) old,MYF(0));
  }
  root->used=root->free=0;
  DBUG_VOID_RETURN;
}


char *sql_strdup_root(MEM_ROOT *root,char *str)
{
  uint len=strlen(str)+1;
  char *pos;
  if ((pos=sql_alloc_root(root,len)))
    memcpy(pos,str,len);
  return pos;
}

char *sql_memdup_root(MEM_ROOT *root,char *str,uint len)
{
  char *pos;
  if ((pos=sql_alloc_root(root,len)))
    memcpy(pos,str,len);
  return pos;
}
