/* Copyright Abandoned 1996 TCX DataKonsult AB & Monty Program KB & Detron HB
   This file is public domain and comes with NO WARRANTY of any kind */

#include "mysys_priv.h"
#include "my_static.h"
#include <errno.h>
#include "errors.h"

static void	make_ftype _A((string to,int flag));

	/* Open a file as stream */

FILE *my_fopen(FileName, Flags, MyFlags)
const char *FileName;			/* Path-name of file */
int Flags;				/* Read | write .. */
myf MyFlags;				/* Special flags */
{
  FILE *fd;
  char type[5];
  DBUG_ENTER("my_fopen");
  DBUG_PRINT("my",("Name: '%s'  Flags: %d  MyFlags: %d",
		   FileName, Flags, MyFlags));

  make_ftype(type,Flags);
  if ((fd = fopen(FileName, type)) != 0)
  {
    pthread_mutex_lock(&THR_LOCK_open);
    if ((my_file_info[fileno(fd)].name = (char*)
	 my_strdup(FileName,MyFlags)))
    {
      my_stream_opened++;
      my_file_info[fileno(fd)].type = STREAM_BY_FOPEN;
      pthread_mutex_unlock(&THR_LOCK_open);
      DBUG_PRINT("exit",("stream: %lx",fd));
      DBUG_RETURN(fd);
    }
    pthread_mutex_unlock(&THR_LOCK_open);
    VOID(my_fclose(fd,MyFlags));
    my_errno=ENOMEM;
  }
  else
    my_errno=errno;
  if (MyFlags & (MY_FFNF | MY_FAE | MY_WME))
    my_error(EE_FILENOTFOUND, MYF(ME_BELL+ME_WAITTANG), FileName,my_errno);
  DBUG_RETURN((FILE*) 0);
} /* my_fopen */


	/* Close a stream */

int my_fclose(fd, MyFlags)
FILE *fd;
myf MyFlags;
{
  int err,file;
  DBUG_ENTER("my_fclose");
  DBUG_PRINT("my",("stream: %lx  MyFlags: %d",fd, MyFlags));

  pthread_mutex_lock(&THR_LOCK_open);
  file=fileno(fd);
  if ((err = fclose(fd)) < 0)
  {
    my_errno=errno;
    if (MyFlags & (MY_FAE | MY_WME))
      my_error(EE_BADCLOSE, MYF(ME_BELL+ME_WAITTANG),
	       my_filename(file),errno);
  }
  if (my_file_info[file].type != UNOPEN)
  {
    my_file_info[file].type = UNOPEN;
    my_free(my_file_info[file].name, MYF(0));
    my_stream_opened--;
  }
  pthread_mutex_unlock(&THR_LOCK_open);
  DBUG_RETURN(err);
} /* my_fclose */


	/* Make a stream out of a file handle */

FILE *my_fdopen(Filedes, Flags, MyFlags)
File Filedes;
int Flags;			/* Read | write .. */
myf MyFlags;			/* Special flags */
{
  FILE *fd;
  char type[5];
  DBUG_ENTER("my_fdopen");
  DBUG_PRINT("my",("Fd: %d  Flags: %d  MyFlags: %d",
		   Filedes, Flags, MyFlags));

  make_ftype(type,Flags);
  if ((fd = fdopen(Filedes, type)) == 0)
  {
    my_errno=errno;
    if (MyFlags & (MY_FAE | MY_WME))
      my_error(EE_CANT_OPEN_STREAM, MYF(ME_BELL+ME_WAITTANG),errno);
  }
  else
  {
    pthread_mutex_lock(&THR_LOCK_open);
    if (my_file_info[Filedes].type != UNOPEN)
    {
      my_file_info[Filedes].type = STREAM_BY_FDOPEN;
      my_file_opened--;			/* File is opened with my_open ! */
      my_stream_opened++;
    }
    pthread_mutex_unlock(&THR_LOCK_open);
  }

  DBUG_PRINT("exit",("stream: %lx",fd));
  DBUG_RETURN(fd);
} /* my_fdopen */


	/* Make a filehandler-open-typestring from ordinary inputflags */

static void make_ftype(to,flag)
reg1 string to;
reg2 int flag;
{
#if FILE_BINARY					/* If we have binary-files */
  reg3 int org_flag=flag;
#endif
  flag&= ~FILE_BINARY;				/* remove binary bit */
  if (flag == O_RDONLY)
    *to++= 'r';
  else if (flag == O_WRONLY)
    *to++= 'w';
  else
  {						/* Add '+' after theese */
    if (flag == O_RDWR)
      *to++= 'r';
    else if (flag & O_APPEND)
      *to++= 'a';
    else
      *to++= 'w';				/* Create file */
    *to++= '+';
  }
#if FILE_BINARY					/* If we have binary-files */
  if (org_flag & FILE_BINARY)
    *to++='b';
#endif
  *to='\0';
} /* make_ftype */
